@@ -8,7 +8,6 @@ use percent_encoding::{AsciiSet, CONTROLS};
 /// base set of characters that must be URL encoded
 const BASE_SET: &AsciiSet = &CONTROLS
     .add(b' ')
-    .add(b'/')
     // RFC-3986 §3.3 allows sub-delims (defined in section2.2) to be in the path component.
     // This includes both colon ':' and comma ',' characters.
     // Smithy protocol tests & AWS services percent encode these expected values. Signing
@@ -32,6 +31,13 @@ const BASE_SET: &AsciiSet = &CONTROLS
     .add(b'=')
     .add(b'%');
 
-pub(super) fn percent_encode(value: &str) -> String {
-    percent_encoding::percent_encode(&value.as_bytes(), BASE_SET).to_string()
+const QUERY_SET: &AsciiSet = &BASE_SET.add(b'/');
+const PATH_SET: &AsciiSet = BASE_SET;
+
+pub(super) fn percent_encode_query(value: &str) -> String {
+    percent_encoding::percent_encode(value.as_bytes(), QUERY_SET).to_string()
+}
+
+pub(super) fn percent_encode_path(value: &str) -> String {
+    percent_encoding::percent_encode(value.as_bytes(), PATH_SET).to_string()
 }
